#!/bin/sh
#miniupnpd control script for Routertech 2.92  mstombs Nov 09
#called from binary upnpd which is started by core logic
ME=miniupnpcmd.sh
#full path
FP=/usr/sbin/$ME
IPT=/sbin/iptables
IPTn="$IPT -t nat"
UPS=/etc/ppp/ip-up
DNS=/etc/ppp/ip-down
LGR=/usr/bin/logger
CNF=/var/tmp/miniupnpd.conf
SHB='#!/bin/sh'
LG(){
	echo "$ME: $@"|$LGR -s
}
EX(){
	LG "Error - $1, exiting"
	exit 1
}

[ -z "$3" ]&& EX "invalid params: $@"

WIF="$2"
WIP="$3"

DIVERT(){
#check if entry already there
	ln=$($IPTn -nL PREROUTING --line-numbers|grep UPNP)
	n=${ln%% *}
	if [ "$n" -gt "0" ];then #replace
		$IPTn -R PREROUTING $n -d $WIP -i $WIF -j UPNP
	else #new
#check for DMZ
		ln=$($IPTn -nL PREROUTING --line-numbers|grep all|grep DNAT)
		n=${ln%% *}
		if [ "$n" -gt "0" ];then #insert so goes above DMZ
			$IPTn -I PREROUTING $n -d $WIP -i $WIF -j UPNP
		else
			$IPTn -A PREROUTING -d $WIP -i $WIF -j UPNP
		fi
	fi
}

UNDIVERT(){
#try this - doesn't work if WIP changed
$IPTn -D PREROUTING -d $WIP -i $WIF -j UPNP
#clean up any extra  entries
while ln=$($IPTn -nL PREROUTING --line-numbers|grep UPNP) && n=${ln%% *} && [ "$n" -gt "0" ]
do
	$IPTn -D PREROUTING $n
done
}

UNFORWARD(){
#Remove divert
$IPT -D FORWARD -i $WIF -o ! $WIF -j UPNP
#clean up any extra  entries
while ln=$($IPT -nL FORWARD --line-numbers|grep UPNP) && n=${ln%% *} && [ "$n" -gt "0" ]
do
	$IPT -D FORWARD $n
done
}

case "$1" in

INIT)
#Create nat chain
$IPTn -N UPNP
#Flush chain - in case it already exist
$IPTn -F UPNP
#Add incoming WAN upnp divert
DIVERT
#Create filter chain
$IPT -N UPNP
#Flush filter chain
$IPT -F UPNP
#remove any old entries
UNFORWARD
# Check for default DROP or ACCEPT (DMZ)
ln=$($IPT -nL FORWARD --line-numbers|grep all|grep -v state|grep -v ipa)
n=${ln%% *}
#Add upnp divert
if [ "$n" -gt "0" ];then #insert so goes above DROP/DMZ
	$IPT -I FORWARD $n  -i $WIF -o ! $WIF -j UPNP
else
	$IPT -A FORWARD -i $WIF -o ! $WIF -j UPNP
fi
#Add ip-dn and up scripts to change the primary iptables divert
if ! grep -q $ME $DNS; then
	[ ! -x $DNS ]&&echo "$SHB">$DNS
	cat >>$DNS<<EOF1
$LGR "ip-down $ME remove divert"
$FP DN \$1 \$4
EOF1
fi

if ! grep -q $ME $UPS; then
	[ ! -x $UPS ]&&echo "$SHB">$UPS
	cat >>$UPS<<EOF2
$LGR "ip-up $ME update divert"
$FP UP \$1 \$4
EOF2
fi
chmod +x $DNS $UPS
# add default block on low number ports
if [ ! -r "$CNF" ] || ! grep deny $CNF; then
    echo "deny 0-1024 0.0.0.0/0 0-1024" >>$CNF
fi
LG upnpd initialized
;;

UP)
DIVERT
LG "divert replaced"
;;

DN)
UNDIVERT
LG "divert removed"
;;

EXIT)
#Flush nat chain
$IPTn -F UPNP
#remove primary divert
UNDIVERT
#Delete chain
$IPTn -X UPNP
#Flush filter chain
$IPT -F UPNP
#Remove divert
UNFORWARD
#Delete filter chain
$IPT -X UPNP

#Remove lines from ip-up & down scripts
sed -i "/$ME/d" $DNS
sed -i "/$ME/d" $UPS

LG upnpd uninstalled
;;

*) EX "usage: INIT/EXIT/UP/DN WIF WIP"
;;
esac
exit
